/*
   --------------------------------------------------------------------------------------------------------------------
   Learning Kit Code
   --------------------------------------------------------------------------------------------------------------------
   This is a MFRC522 library example; for further details and other examples see: https://github.com/miguelbalboa/rfid

   This example showing a complete Door Access Control System

 * **Stores Information on EEPROM**
   Information stored on non volatile Arduino's EEPROM memory to preserve Users' tag and Master Card. No Information lost
   if power lost. EEPROM has unlimited Read cycle but roughly 100,000 limited Write cycle.

 * **Security**
   To keep it simple we are going to use Tag's Unique IDs. It's simple and not hacker proof.

   Typical pin layout used:
   -----------------------------------------------------------------------------------------
               MFRC522      Arduino       Arduino   Arduino    Arduino          Arduino
               Reader/PCD   Uno/101       Mega      Nano v3    Leonardo/Micro   Pro Micro
   Signal      Pin          Pin           Pin       Pin        Pin              Pin
   -----------------------------------------------------------------------------------------
   RST/Reset   RST          9             5         D9         RESET/ICSP-5     RST
   SPI SS      SDA(SS)      10            53        D10        10               10
   SPI MOSI    MOSI         11 / ICSP-4   51        D11        ICSP-4           16
   SPI MISO    MISO         12 / ICSP-1   50        D12        ICSP-1           14
   SPI SCK     SCK          13 / ICSP-3   52        D13        ICSP-3           15
*/


/*EEPROM and RFID Reader Setup---*/
#include <EEPROM.h>     // We are going to read and write PICC's UIDs from/to EEPROM
#include <SPI.h>        // RC522 Module uses SPI protocol
#include <MFRC522.h>  // Library for Mifare RC522 Devices

/*---Servo Setup---*/
#include <Servo.h>
Servo myservo;
int posServo = 0; // set pos Servo by default
int switchState = 0; // set state for button detecting the presence of object

/*---NeoPixel Setup---*/
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h>
#endif
// Which pin on the Arduino is connected to the NeoPixels?
#define PIN            6
// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS      9
// When we setup the NeoPixel library, we tell it how many pixels, and which pin to use to send signals.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);


uint8_t successRead;    // Variable integer to keep if we have Successful Read from Reader
byte storedCard[4];   // Stores an ID read from EEPROM
byte readCard[4];   // Stores scanned ID read from RFID Module

// Create MFRC522 instance.
constexpr uint8_t RST_PIN = 9;     // Configurable, see typical pin layout above
constexpr uint8_t SS_PIN = 10;     // Configurable, see typical pin layout above

MFRC522 mfrc522(SS_PIN, RST_PIN);

///////////////////////////////////////// Setup ///////////////////////////////////
void setup() {

  //NeoPixel Configuration
  pixels.begin(); // This initializes the NeoPixel library.

  //Servo Configuration
  pinMode(2, INPUT);
  myservo.attach(3);

  //Arduino Pin Configuration

  //Be careful how relay circuit behave on while resetting or power-cycling your Arduino
  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // LED 0 is off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  //Protocol Configuration
  Serial.begin(9600);  // Initialize serial communications with PC
  SPI.begin();           // MFRC522 Hardware uses SPI protocol
  mfrc522.PCD_Init();    // Initialize MFRC522 Hardware

  //If you set Antenna Gain to Max it will increase reading distance
  //mfrc522.PCD_SetAntennaGain(mfrc522.RxGain_max);

  Serial.println(F("Access Control Example v0.1"));   // For debugging purposes
  ShowReaderDetails();  // Show details of PCD - MFRC522 Card Reader details

  Serial.println("");
  Serial.println(F("-------------------"));
  Serial.println(F("Everything is ready"));
  Serial.println(F("Waiting PICCs to be scanned"));
  cycleLeds();    // Everything ready lets give user some feedback by cycling leds
}


///////////////////////////////////////// Main Loop ///////////////////////////////////
void loop () {

  do {

    //check for RFID reading
    successRead = getID();  // sets successRead to 1 when we get read from reader otherwise 0

    //Check whether the learning kit is back on the shelf
    switchState = digitalRead(2); // check what is the state of the button detecting the learning kit
    if ( switchState == 1) { // check if the button detecting the learning kit is pressed, if it is :
      Serial.println(F("Learning kit is detected"));
      delay(10);
      myservo.write(90); // locks the learning kit
      delay(15);
    }
    
    
    normalModeOn();     // Normal mode, NeoPixel is blue
    
  }
  while (!successRead);   //the program will not go further while you are not getting a successful read
   
      if ( findID(readCard) ) { // If not, see if the card is in the EEPROM
        Serial.println(F("Learning Kit unlocked, enjoy!"));
        granted(300);         // Open the door lock for 300 ms
      }
      else {      // If not, show that the ID was not valid
        Serial.println(F("Talk to an Instructor to access this Learning Kit, Thanks."));
        denied();
      }
}

/////////////////////////////////////////  Access Granted    ///////////////////////////////////
void granted ( uint16_t setDelay) {
  pixels.setPixelColor(0, pixels.Color(0, 220, 0)); // green color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  //delay(1000);
  myservo.write(0); // Unlock learning kit!
  delay(5000); // Hold learning kit unlocked for given seconds

  //Check whether the learning kit is still on the shelf
  switchState = digitalRead(2); // check what is the state of the button detecting the learning kit
  if ( switchState == 1) { // check if the button detecting the learning kit is pressed, if it is :
    Serial.println(F("Learning kit is still detected"));
    delay(1000);
    myservo.write(90); // locks the learning kit
    delay(15);
  }
  delay(1000);            // Hold green LED on for a second
}

///////////////////////////////////////// Access Denied  ///////////////////////////////////
void denied() {
  pixels.setPixelColor(0, pixels.Color(220, 0, 0)); // red color.
  pixels.show(); // This sends the updated pixel color to the hardware.
  delay(2000);
}


///////////////////////////////////////// Get PICC's UID ///////////////////////////////////
uint8_t getID() {
  // Getting ready for Reading PICCs
  if ( ! mfrc522.PICC_IsNewCardPresent()) { //If a new PICC placed to RFID reader continue
    return 0;
  }
  if ( ! mfrc522.PICC_ReadCardSerial()) {   //Since a PICC placed get Serial and continue
    return 0;
  }
  // There are Mifare PICCs which have 4 byte or 7 byte UID care if you use 7 byte PICC
  // I think we should assume every PICC as they have 4 byte UID
  // Until we support 7 byte PICCs
  Serial.println(F("Scanned PICC's UID:"));
  for ( uint8_t i = 0; i < 4; i++) {  //
    readCard[i] = mfrc522.uid.uidByte[i];
    Serial.print(readCard[i], HEX);
  }
  Serial.println("");
  mfrc522.PICC_HaltA(); // Stop reading
  return 1;
}

void ShowReaderDetails() {
  // Get the MFRC522 software version
  byte v = mfrc522.PCD_ReadRegister(mfrc522.VersionReg);
  Serial.print(F("MFRC522 Software Version: 0x"));
  Serial.print(v, HEX);
  if (v == 0x91)
    Serial.print(F(" = v1.0"));
  else if (v == 0x92)
    Serial.print(F(" = v2.0"));
  else
    Serial.print(F(" (unknown),probably a chinese clone?"));
  Serial.println("");
  // When 0x00 or 0xFF is returned, communication probably failed
  if ((v == 0x00) || (v == 0xFF)) {
    Serial.println(F("WARNING: Communication failure, is the MFRC522 properly connected?"));
    Serial.println(F("SYSTEM HALTED: Check connections."));
    // Visualize system is halted
    pixels.setPixelColor(0, pixels.Color(220, 0, 0)); // red color.
    pixels.show(); // This sends the updated pixel color to the hardware.
    while (true); // do not go further
  }
}

///////////////////////////////////////// Cycle Leds (Program Mode) ///////////////////////////////////
void cycleLeds() {
  //digitalWrite(redLed, LED_OFF);  // Make sure red LED is off
  //digitalWrite(greenLed, LED_ON);   // Make sure green LED is on
  //digitalWrite(blueLed, LED_OFF);   // Make sure blue LED is off
  pixels.setPixelColor(0, pixels.Color(0, 220, 0)); // green color.
  pixels.show(); // This sends the updated pixel color to the hardware.
  delay(200);
  //digitalWrite(redLed, LED_OFF);  // Make sure red LED is off
  //digitalWrite(greenLed, LED_OFF);  // Make sure green LED is off
  //digitalWrite(blueLed, LED_ON);  // Make sure blue LED is on
  pixels.setPixelColor(0, pixels.Color(0, 0, 220)); // blue color.
  pixels.show(); // This sends the updated pixel color to the hardware.
  delay(200);
  //digitalWrite(redLed, LED_ON);   // Make sure red LED is on
  //digitalWrite(greenLed, LED_OFF);  // Make sure green LED is off
  //digitalWrite(blueLed, LED_OFF);   // Make sure blue LED is off
  pixels.setPixelColor(0, pixels.Color(220, 0, 0)); // blue color.
  pixels.show(); // This sends the updated pixel color to the hardware.
  delay(200);
}

//////////////////////////////////////// Normal Mode Led  ///////////////////////////////////
void normalModeOn () {
  //digitalWrite(blueLed, LED_ON);  // Blue LED ON and ready to read card
  //digitalWrite(redLed, LED_OFF);  // Make sure Red LED is off
  //digitalWrite(greenLed, LED_OFF);  // Make sure Green LED is off
  pixels.setPixelColor(0, pixels.Color(0, 0, 220)); // blue color.
  pixels.show(); // This sends the updated pixel color to the hardware.
  //digitalWrite(relay, HIGH);    // Make sure Door is Locked
}

//////////////////////////////////////// Read an ID from EEPROM //////////////////////////////
void readID( uint8_t number ) {
  uint8_t start = (number * 4 ) + 2;    // Figure out starting position
  for ( uint8_t i = 0; i < 4; i++ ) {     // Loop 4 times to get the 4 Bytes
    storedCard[i] = EEPROM.read(start + i);   // Assign values read from EEPROM to array
  }
}

///////////////////////////////////////// Check Bytes   ///////////////////////////////////
bool checkTwo ( byte a[], byte b[] ) {
  for ( uint8_t k = 0; k < 4; k++ ) {   // Loop 4 times
    if ( a[k] != b[k] ) {     // IF a != b then false, because: one fails, all fail
      return false;
    }
  }
  return true;
}

///////////////////////////////////////// Find Slot   ///////////////////////////////////
uint8_t findIDSLOT( byte find[] ) {
  uint8_t count = EEPROM.read(0);       // Read the first Byte of EEPROM that
  for ( uint8_t i = 1; i <= count; i++ ) {    // Loop once for each EEPROM entry
    readID(i);                // Read an ID from EEPROM, it is stored in storedCard[4]
    if ( checkTwo( find, storedCard ) ) {   // Check to see if the storedCard read from EEPROM
      // is the same as the find[] ID card passed
      return i;         // The slot number of the card
    }
  }
}

///////////////////////////////////////// Find ID From EEPROM   ///////////////////////////////////
bool findID( byte find[] ) {
  uint8_t count = EEPROM.read(0);     // Read the first Byte of EEPROM that
  for ( uint8_t i = 1; i < count; i++ ) {    // Loop once for each EEPROM entry
    readID(i);          // Read an ID from EEPROM, it is stored in storedCard[4]
    if ( checkTwo( find, storedCard ) ) {   // Check to see if the storedCard read from EEPROM
      return true;
    }
    else {    // If not, return false
    }
  }
  return false;
}

///////////////////////////////////////// Write Success to EEPROM   ///////////////////////////////////
// Flashes the green LED 3 times to indicate a successful write to EEPROM
void successWrite() {

  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // led off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(greenLed, LED_ON);   // Make sure green LED is on
  pixels.setPixelColor(0, pixels.Color(0, 220, 0)); // green color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(greenLed, LED_OFF);  // Make sure green LED is off
  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // led off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(greenLed, LED_ON);   // Make sure green LED is on
  pixels.setPixelColor(0, pixels.Color(0, 220, 0)); // green color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(greenLed, LED_OFF);  // Make sure green LED is off
  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // led off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(greenLed, LED_ON);   // Make sure green LED is on
  pixels.setPixelColor(0, pixels.Color(0, 220, 0)); // green color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
}

///////////////////////////////////////// Write Failed to EEPROM   ///////////////////////////////////
// Flashes the red LED 3 times to indicate a failed write to EEPROM
void failedWrite() {
  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // led off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(redLed, LED_ON);   // Make sure red LED is on
  pixels.setPixelColor(0, pixels.Color(220, 0, 0)); // red color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(redLed, LED_OFF);  // Make sure red LED is off
  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // led off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(redLed, LED_ON);   // Make sure red LED is on
  pixels.setPixelColor(0, pixels.Color(220, 0, 0)); // red color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(redLed, LED_OFF);  // Make sure red LED is off
  pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // led off.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
  //digitalWrite(redLed, LED_ON);   // Make sure red LED is on
  pixels.setPixelColor(0, pixels.Color(220, 0, 0)); // red color.
  pixels.show(); // This sends the updated pixel color to the hardware.

  delay(200);
}

